//
//  HomeViewController.m
//  IOS Learning Plaform
//
//  Created by Rubén Domínguez on 09/10/13.
//  Copyright (c) 2013 TheAgileMonkeys. All rights reserved.
//

#import "HomeViewController.h"
#import "VideosProgressBar.h"
#import "VideoPlayerViewController.h"
#import "voteViewController.h"
#import <Parse/Parse.h>
#import "Lesson.h"
#import "FinishCourseViewController.h"

@interface HomeViewController () <VideoPlayerViewController>
    @property (weak, nonatomic) IBOutlet VideosProgressBar *videoProgressBarView;
    @property (weak, nonatomic) IBOutlet UILabel *HomeTitle;
    @property (weak, nonatomic) IBOutlet UIImageView *placeholder;
    @property (weak, nonatomic) IBOutlet UILabel *startVideoLabel;
    @property (weak, nonatomic) IBOutlet UILabel *videoWachedLabel;
    @property (weak, nonatomic) IBOutlet UILabel *percentLabel;
    @property (strong, nonatomic) PFObject *video;
    @property (strong, nonatomic) Lesson *lesson;
    @property (nonatomic, weak) NSManagedObjectContext *moc;
@end

@implementation HomeViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(loadData)
                                                 name:@"reloadData"
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(loadPage)
                                                 name:@"loadPageandreloadData"
                                               object:nil];
    
    [self.startVideoLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:15]];
    [self.videoWachedLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:15]];
    self.videoWachedLabel.text =  NSLocalizedString(@"videosWatchedSentence", nil);
    
    [self.percentLabel setFont:[UIFont fontWithName:@"Aleo-Light" size:60]];
    self.percentLabel.textColor = navbarBackground;
    
    [self.HomeTitle setFont:[UIFont fontWithName:@"Aleo-Regular" size:20]];
    self.HomeTitle.textColor = navbarBackground;
    self.placeholder.hidden = NO;
}


- (void) viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    self.navigationController.navigationBar.topItem.title = NSLocalizedString(@"homePageTitle", nil);
    [self loadData];
}

#pragma mark private methods

- (void)playVideo:(UITapGestureRecognizer *)sender
{
    if(self.video){
        VideoPlayerViewController * videoPlayer = [[VideoPlayerViewController alloc] init];
        videoPlayer.delegate = self;
        videoPlayer.video_url = self.video[@"filename"];
        [self presentViewController:videoPlayer animated:YES completion:nil];
    }
    else{
        [ErrorMessageHelper showErrorMessageWith:NSLocalizedString(@"NetworkError", nil)];          
    }
}

- (void) loadData{
    self.lesson = [Lesson getActualLesson];
    if (self.lesson) {
        PFQuery *query = [PFQuery queryWithClassName:@"Video"];
        [query getObjectInBackgroundWithId:self.lesson.video block:^(PFObject *video, NSError *error) {
            if (!error) {
                self.video = video;
                // Do any additional setup after loading the view.
                NSString *field = ([[[NSLocale preferredLanguages] objectAtIndex:0] isEqualToString:@"es"]) ? @"spanish_title" : @"english_title";
                self.HomeTitle.text = self.video[field];
                self.startVideoLabel.text = NSLocalizedString(@"startVideoSentence", nil);
                self.placeholder.userInteractionEnabled = YES;
                UITapGestureRecognizer *oneTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(playVideo:)];
                [self.placeholder addGestureRecognizer:oneTap];
            }
        }];
    }
    else{
            //mostrar el alertazo y mandarlo al examen
        FinishCourseViewController *finishCoursenotice = [self.storyboard instantiateViewControllerWithIdentifier:@"finishCourseVC"];
        finishCoursenotice.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        [self presentViewController:finishCoursenotice animated:YES completion:nil];

        
    }

    NSUInteger n_viewed = [Lesson videosViewed];
    NSNumber *percent = @(n_viewed *100 / n_videos);
    self.percentLabel .text =[[NSString alloc] initWithFormat:@"%@%%", percent];
    [self.videoProgressBarView setNeedsDisplay];
}

- (void) loadPage{
    self.tabBarController.selectedIndex = 0;
    [self loadData];
}


- (void) videoFinishCallback{
    
    if (![self.lesson.done boolValue])
    {
        [self.lesson markLikeDone];
        NSString *strDate = [NSDateFormatter localizedStringFromDate:[NSDate date]
                                                           dateStyle:NSDateFormatterMediumStyle
                                                           timeStyle:NSDateFormatterNoStyle];

        NSDictionary *dimensions = @{
                                    @"userMail": [[NSUserDefaults standardUserDefaults] valueForKey:@"userMail"],
                                    @"userId": [[NSUserDefaults standardUserDefaults] valueForKey:@"userId"],
                                    @"VideoId": self.lesson.video,
                                    @"date": strDate
                                    };
        // Send the dimensions to Parse along with the 'search' event
        [PFAnalytics trackEvent:@"VideoViews" dimensions:dimensions];
        voteViewController *voteController = [self.storyboard instantiateViewControllerWithIdentifier:(@"voteViewController")];
        voteController.video = self.video;
        voteController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        [self presentViewController:voteController animated:YES completion:nil];
        self.tabBarController.selectedIndex = 1;
    }
}

@end

