//
//  AMKeyboardsAfraidScrollView.m
//  CityNostra
//
//  Created by Rubén Domínguez Falcón on 31/05/13.
//  Copyright (c) 2013 TheAgileMonkeys. All rights reserved.
//

#import "AMKeyboardsAfraidScrollView.h"

@interface AMKeyboardsAfraidScrollView()

//@property (nonatomic) BOOL priorUserInteraction; TO DO.

@end

@implementation AMKeyboardsAfraidScrollView


- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        [self registerForKeyboardNotifications];
    }
    return self;
}

- (void)awakeFromNib //For storyboards and nibs.
{
    [self registerForKeyboardNotifications];
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}


- (void)registerForKeyboardNotifications
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardDidAppear:)
                                                 name:UIKeyboardDidShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillDisappear:)
                                                 name:UIKeyboardWillHideNotification object:nil];
}

- (CGPoint)getTheLowestPointOfActiveField
{
    CGPoint lowestPoint = CGPointZero;
    
    for (UIView *aView in self.subviews)
    {
        if ([aView isKindOfClass:[UITextView class]] || [aView isKindOfClass:[UITextField class]])
        {
            CGPoint aViewLowerPoint = CGPointMake(aView.frame.origin.x, aView.frame.origin.y + aView.frame.size.height);
            
            if ([aView isFirstResponder] && (aViewLowerPoint.y > lowestPoint.y))
                lowestPoint = aViewLowerPoint;
        }
    }
    
    return lowestPoint;
}

// Called when the UIKeyboardDidShowNotification is sent.
- (void)keyboardDidAppear:(NSNotification*)aNotification
{
    NSDictionary* info = [aNotification userInfo];
    CGSize keyboardSize = [[info objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;
    UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.0, 0.0, keyboardSize.height, 0.0);
    [self setContentInset:contentInsets];
    [self setScrollIndicatorInsets:contentInsets];
    
    //Get the lowest active field:
    CGPoint lowestPoint = [self getTheLowestPointOfActiveField];
    if (!CGPointEqualToPoint(lowestPoint, CGPointZero))
    {
        CGRect aRect = self.frame;
        aRect.size.height -= keyboardSize.height;
        if (!CGRectContainsPoint(aRect, lowestPoint))
        {
            // If active text field is hidden by keyboard, scroll it so it's visible
            CGPoint scrollPoint = CGPointMake(0, lowestPoint.y - keyboardSize.height - 20);
            [self setContentOffset:scrollPoint animated:YES];
        }
    }
}

// Called when the UIKeyboardWillHideNotification is sent
- (void)keyboardWillDisappear:(NSNotification*)aNotification
{
    UIEdgeInsets contentInsets = UIEdgeInsetsZero;
    [self setContentInset:contentInsets];
    [self setScrollIndicatorInsets:contentInsets];
    
}

- (void)touchesEnded: (NSSet *) touches withEvent: (UIEvent *) event
{
    // If not dragging, send event to next responder
    if (!self.dragging)
        [self.nextResponder touchesEnded: touches withEvent:event];
    else
        [super touchesEnded: touches withEvent: event];
}

@end
