//
//  CourseManager.m
//  PFC
//
//  Created by Rubén Domínguez on 2/8/16.
//  Copyright © 2016 TheAgileMonkeys. All rights reserved.
//

#import "CourseManager.h"
#import <Parse/Parse.h>

@interface CourseManager ()
@property (nonatomic, copy) NSArray *videos;
@property (nonatomic, readwrite, copy) NSArray *lessons;


@end

@implementation CourseManager

+ (instancetype)sharedInstance
{
    static dispatch_once_t once;
    static CourseManager *sharedInstance;
    
    dispatch_once(&once, ^
                  {
                      sharedInstance = [self new];
                      sharedInstance.currentUser = [User new];
                  });
    
    return sharedInstance;
}

- (void)configureCourse
{
    [Lesson getLessonsForCourseWithCompletionBlock:^(NSArray *lessons, NSError *error) {
        if(error) {
            [ErrorMessageHelper showErrorMessageWith:@"Can't fetch the lessons for course"];
        }
        else {
            self.lessons = lessons;
        }
    }];
    
    [Lesson getVideosForLessonsWithCompletionBlock:^(NSArray *videos, NSError *error) {
        if(error) {
            [ErrorMessageHelper showErrorMessageWith:@"Can't fetch the videos for course"];
        }
        else {
            self.videos = videos;
        }
    }];
    
    if ([self isUserSigned])
    {
        self.currentUser.modelId = [[NSUserDefaults standardUserDefaults] valueForKey:@"userId"];
        
        [self.currentUser fetchUserInformationWithCompletion:nil];
    }
}

- (void)updateUserInformationWithCompletion:(void (^)(NSError *error))completion
{
    [self.currentUser updateUserInformationWithCompletion:^(User *currentUser, NSError *error) {
        
        if (error) {
            completion(error);
        }
        else {
            
            if(![self isUserSigned]) {
                [[NSUserDefaults standardUserDefaults]setValue:self.currentUser.modelId forKey:@"userId"];
                [[NSUserDefaults standardUserDefaults]setBool:YES forKey:@"isUserRegistered"];
                [[NSUserDefaults standardUserDefaults]setValue:currentUser.email forKey:@"userMail"];
                [[NSUserDefaults standardUserDefaults] synchronize];
            }
            [self prepareLessonsForCurrenStudent];
            completion(nil);
        }
    }];
}

- (BOOL)isUserSigned
{
    return [[NSUserDefaults standardUserDefaults] valueForKey:@"isUserRegistered"] != nil;
}

- (BOOL)isTestFinished
{
    return [[NSUserDefaults standardUserDefaults] valueForKey:@"isTestFinished"] != nil;
}



#pragma mark - Private Methods

- (void) setNotification:(NSDate *)date :(NSString*)message{
    UILocalNotification* localNotification = [[UILocalNotification alloc] init];
    localNotification.fireDate = date;
    localNotification.timeZone = [NSTimeZone defaultTimeZone];
    localNotification.alertBody = message;
    localNotification.hasAction = YES;
    localNotification.soundName = UILocalNotificationDefaultSoundName;
    localNotification.applicationIconBadgeNumber = localNotification.applicationIconBadgeNumber + 1;
    [[UIApplication sharedApplication] scheduleLocalNotification:localNotification];
}

- (void) prepareLessonsForCurrenStudent{
    

    NSString *field = ([[[NSLocale preferredLanguages] objectAtIndex:0] isEqualToString:@"es"]) ? @"spanish_notification" : @"english_notification";
   
    UIApplication* app = [UIApplication sharedApplication];
    NSArray *oldNotifications = [app scheduledLocalNotifications];
    
    if (oldNotifications) {
        [app cancelAllLocalNotifications];
    }
    
    NSCalendar *calendar = [NSCalendar currentCalendar];
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init] ;
    [formatter setDateFormat:@"HH:mm"];
    
    NSDateComponents *components = [calendar components:(NSHourCalendarUnit | NSMinuteCalendarUnit) fromDate:[formatter dateFromString:self.currentUser.videoAvaliableTime]];
    
    NSInteger hour = [components hour];
    NSInteger minute = [components minute];
    NSInteger index = 0;

    for (Lesson *lesson in self.lessons) {
            NSCalendar *gregorian = [[NSCalendar alloc]initWithCalendarIdentifier:NSGregorianCalendar];
            NSDateComponents *dateComponents = [gregorian components:(NSYearCalendarUnit | NSMonthCalendarUnit | NSDayCalendarUnit) fromDate: lesson.date];
        [dateComponents setHour:hour];
        [dateComponents setMinute:minute];
        
        NSDate *newDate = [gregorian dateFromComponents:dateComponents];
        [lesson updateWithDate: [newDate dateByAddingTimeInterval:timeBetweenVideos*index] andCompletion:nil];
        index ++;
        PFObject * video = [self getVideoFromLesson:lesson];
        [self setNotification:lesson.date :video[field]];
    }
}

-(PFObject *) getVideoFromLesson:(Lesson *) lesson{
    PFObject *video;
    for (video in self.videos) {
        if ([video.objectId isEqualToString:lesson.video]) {
            return video;
        }
    }
    
    return nil;
}

@end
