//
//  HistoryViewController.m
//  IOS Learning Plaform
//
//  Created by Rubén Domínguez on 25/10/13.
//  Copyright (c) 2013 TheAgileMonkeys. All rights reserved.
//

#import "HistoryViewController.h"
#import "VideoPlayerViewController.h"
#import "Lesson.h"
#import <Parse/Parse.h>

@interface HistoryViewController ()<NSFetchedResultsControllerDelegate, UITableViewDelegate, VideoPlayerViewController>
@property (nonatomic, weak) NSManagedObjectContext *moc;
@property (strong, nonatomic) NSFetchedResultsController *fetchedResultsController;
@property (strong, nonatomic) NSArray *videos;



@end

@implementation HistoryViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.moc =  [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
   
    
    NSError *error = nil;
    if (![self.fetchedResultsController performFetch:&error])
    {
        NSLog(@"ERROR = %@", [error localizedDescription]);
    }
    PFQuery *query = [PFQuery queryWithClassName:@"Video"];
    [query findObjectsInBackgroundWithBlock:^(NSArray *objects, NSError *error) {
        if(!error){
            self.videos = objects;
            [self.tableView reloadData];
        }
        
    }];
}

- (void) viewDidLayoutSubviews{
}
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void) viewWillAppear:(BOOL)animated{
    
    self.navigationController.navigationBar.topItem.title = NSLocalizedString(@"historyPageTitle", nil);
    self.fetchedResultsController = nil;
    NSError *error = nil;
    if (![self.fetchedResultsController performFetch:&error])
    {
        NSLog(@"ERROR = %@", [error localizedDescription]);
    }
    [self.tableView reloadData];
}


#pragma mark - Table view data source
- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return 0.0f;
}
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return [[self.fetchedResultsController sections] count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    id <NSFetchedResultsSectionInfo> sectionInfo = [self.fetchedResultsController sections][section];
    return [sectionInfo numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"historyCell" forIndexPath:indexPath];
    
    // Configure the cell...
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}


// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return NO;
}



- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    Lesson *object = [self.fetchedResultsController objectAtIndexPath:indexPath];
   ;
    PFObject *video = [self getVideoFromLesson:object];
    if (video) {
   
        VideoPlayerViewController * videoPlayer = [[VideoPlayerViewController alloc] init];
        videoPlayer.delegate = self;
        videoPlayer.video_url = video[@"filename"];
        videoPlayer.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        [self presentViewController:videoPlayer animated:YES completion:nil];
    }
}

#pragma mark - Fetched results controller

- (NSFetchedResultsController *)fetchedResultsController
{
    if (_fetchedResultsController != nil) {
        return _fetchedResultsController;
    }
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Lesson" inManagedObjectContext:self.moc];
    [fetchRequest setEntity:entity];
    NSDate *endDate = [[NSDate date] dateByAddingTimeInterval:-timeBetweenVideos];
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"(date < %@)", endDate];
    [fetchRequest setPredicate:predicate];

    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:7];
    
    // Edit the sort key as appropriate.
    
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"date" ascending:YES];

    NSArray *sortDescriptors = @[sortDescriptor];
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:self.moc sectionNameKeyPath:nil cacheName:nil];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
	NSError *error = nil;
	if (![self.fetchedResultsController performFetch:&error]) {
        // Replace this implementation with code to handle the error appropriately.
        // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
	    abort();
	}
    
    return _fetchedResultsController;
}

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
           atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type
{
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    UITableView *tableView = self.tableView;
    
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView endUpdates];
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    if(self.videos){
        Lesson *lesson = [self.fetchedResultsController objectAtIndexPath:indexPath];
        UIImageView *imageView;
        NSString *strIndex = [[NSString alloc] initWithFormat:@"%d",indexPath.row +1];
        NSMutableAttributedString *strTitleFormat;
        PFObject *video = [self getVideoFromLesson:lesson];
        
        NSString *field = ([[[NSLocale preferredLanguages] objectAtIndex:0] isEqualToString:@"es"]) ? @"spanish_title" : @"english_title";
            
        strTitleFormat = [[NSMutableAttributedString alloc] initWithString:[strIndex stringByAppendingString:[NSString stringWithFormat:@"  %@",video[field]]]];
    
        [strTitleFormat addAttribute: NSFontAttributeName value:[UIFont fontWithName:@"Aleo-Regular" size:17.0] range: NSMakeRange(0, strTitleFormat.length)];
    
        if ([lesson.done boolValue]) {
            [strTitleFormat addAttribute: NSForegroundColorAttributeName value: backgroudLabel range: NSMakeRange(0, strTitleFormat.length)];
            imageView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"play_history2.png"]];
        
        
        } else {
            [strTitleFormat addAttribute: NSForegroundColorAttributeName value: navbarBackground range: NSMakeRange(0, strIndex.length)];
            [strTitleFormat addAttribute: NSFontAttributeName value:[UIFont fontWithName:@"Aleo-Bold" size:17.0] range: NSMakeRange(0, strIndex.length)];
            imageView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"play_history1.png"]];

        }
        [cell.textLabel setAttributedText:strTitleFormat];
        cell.textLabel.numberOfLines = 0;
        cell.accessoryView = imageView;
    }
}

#pragma mark private methods
-(PFObject *) getVideoFromLesson:(Lesson *) lesson{
    PFObject *video;
    for (video in self.videos) {
        if ([video.objectId isEqualToString:lesson.video]) {
            return video;
        }
    }
    return nil;
}

@end
