//
//  Lesson.m
//  IOS Learning Plaform
//
//  Created by Rubén on 28/11/13.
//  Copyright (c) 2013 TheAgileMonkeys. All rights reserved.
//

#import "Lesson.h"
#import <Parse/Parse.h>


@implementation Lesson

@dynamic date;
@dynamic video;
@dynamic done;

+ (void)getLessonsForCourseWithCompletionBlock:(void (^)(NSArray *lessons, NSError *error))completion
{
    NSMutableArray *lessons = [NSMutableArray array];
    NSManagedObjectContext *moc = [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    NSEntityDescription *entityDescriptor = [NSEntityDescription entityForName:@"Lesson" inManagedObjectContext: moc];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entityDescriptor];
    NSError *error;
    
    if ([moc countForFetchRequest:request error:&error] > 0)
    {

        lessons =  [NSMutableArray arrayWithArray:[moc executeFetchRequest:request error:&error]];
        
        completion(lessons, error);

    }
    else
    {
        PFQuery *query = [PFQuery queryWithClassName:@"Video"];
        
        [query findObjectsInBackgroundWithBlock:^(NSArray *videos, NSError *error) {

            if(!error)
            {
                PFObject *video;
                NSError *localError = nil;

                for (int i = 0; i < [videos count]; i++) {
                    Lesson *new_lesson = [[Lesson alloc] initWithEntity:entityDescriptor insertIntoManagedObjectContext:moc];
                    video = videos[i];
                    new_lesson.video = video.objectId;
                    new_lesson.date = [[NSDate date] dateByAddingTimeInterval:timeBetweenVideos*i];
                    [moc save:&localError];
                    
                    if(!localError)
                    {
                        [lessons addObject:new_lesson];
                    }
                    else {
                        break;
                    }
                }
                
                completion(lessons, localError);
            }
            else
            {
                completion(lessons, error);

            }
        }];
         
    }
}

+ (void)getVideosForLessonsWithCompletionBlock:(void (^)(NSArray *, NSError *))completion
{
    
    PFQuery *query = [PFQuery queryWithClassName:@"Video"];
    
    [query findObjectsInBackgroundWithBlock:^(NSArray *videos, NSError *error) {
        completion(videos, error);
    }];
}

- (void)updateWithDate:(NSDate *)date andCompletion:(void (^)(NSError *error))completion
{
    self.date = date;
    
    NSManagedObjectContext *moc = [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    NSError *localError = nil;
    [moc save:&localError];
    
    if (completion) {
        completion(localError);
    }
}

+ (BOOL) hasLessonsAvaliable{
    NSDate *startDate = [[NSDate date] dateByAddingTimeInterval:-timeBetweenVideos];
    
    NSManagedObjectContext *moc = [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    NSEntityDescription *entityDescription = [NSEntityDescription
                                              entityForName:@"Lesson" inManagedObjectContext:moc];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entityDescription];
    
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"(date >= %@)", startDate];
    [request setPredicate:predicate];
    
    NSError *error;
    NSUInteger nLessons = [moc countForFetchRequest:request error:&error];
    
    return nLessons > 0;
}

+ (BOOL) hasLessonNow
{
    NSDate *startDate = [[NSDate date] dateByAddingTimeInterval:-timeBetweenVideos];
    NSDate *endDate = [startDate dateByAddingTimeInterval:timeBetweenVideos*2];
    
    NSManagedObjectContext *moc = [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    NSEntityDescription *entityDescription = [NSEntityDescription
                                              entityForName:@"Lesson" inManagedObjectContext:moc];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entityDescription];
    
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"(date >= %@ AND date < %@)", startDate, endDate];
    [request setPredicate:predicate];
    NSError *error;
    
    //    NSArray *lessons = [moc executeFetchRequest:request error:&error];
    NSUInteger nLessons = [moc countForFetchRequest:request error:&error];
    
    
    return nLessons > 0;
}

+ (Lesson *) getActualLesson
{
    
    NSManagedObjectContext *moc = [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    NSEntityDescription *entityDescription = [NSEntityDescription
                                              entityForName:@"Lesson" inManagedObjectContext: moc];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entityDescription];
    
    NSError *error;
    Lesson *lesson;
    
    if ([self hasLessonsAvaliable] && ![self hasLessonNow]) {
        NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"date" ascending:YES];
        [request setSortDescriptors:@[sortDescriptor]];
        lesson = [moc executeFetchRequest:request error:&error].firstObject;

    }
    else{
        NSDate *startDate = [[NSDate date] dateByAddingTimeInterval: - timeBetweenVideos];
        NSDate *endDate = [startDate dateByAddingTimeInterval:2 * timeBetweenVideos];
        NSPredicate *predicate = [NSPredicate predicateWithFormat:@"(date >= %@) AND (date < %@)", startDate, endDate];
        [request setPredicate:predicate];
        lesson = [moc executeFetchRequest:request error:&error].firstObject;
    }
    
    return lesson;
}


+ (NSUInteger) videosViewed
{
    
    NSManagedObjectContext *moc = [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    NSEntityDescription *entityDescription = [NSEntityDescription
                                              entityForName:@"Lesson" inManagedObjectContext: moc];

    NSError *error;
    NSFetchRequest *viewed_videos = [[NSFetchRequest alloc] init];
    [viewed_videos setEntity:entityDescription];
    NSPredicate *viewed_predicate = [NSPredicate predicateWithFormat:@"done == '1'"];
    [viewed_videos setPredicate:viewed_predicate];
    
    return [moc countForFetchRequest:viewed_videos error:&error];
}

- (void) markLikeDone
{
    NSManagedObjectContext *moc = [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    
    NSError *error;
    self.done = @YES;
    [moc save:&error];
    
}

#pragma mark - Private Methods

@end
