//
//  RegisterViewController.m
//  IOS Learning Plaform
//
//  Created by Rubén Domínguez Falcón on 18/09/13.
//  Copyright (c) 2013 TheAgileMonkeys. All rights reserved.
//

#import "SignUpViewController.h"
#import "AMKeyboardsAfraidScrollView.h"
#import "ActionSheetPicker.h"

@interface SignUpViewController () <UITextFieldDelegate, UIScrollViewDelegate>
@property (weak, nonatomic) IBOutlet UINavigationBar *navBar;
@property (weak, nonatomic) IBOutlet UILabel *nameTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *occupationTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *birthTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *organizationTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *emailTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *genderTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *timeVideoTitleLabel;

@property (nonatomic, weak) NSManagedObjectContext *moc;

@property (nonatomic, strong) NSArray *occupations;
@property (nonatomic, assign) NSInteger selectedIndex;
@property (weak, nonatomic) IBOutlet AMKeyboardsAfraidScrollView *formScrollableView;
@property (weak, nonatomic) IBOutlet UIView *formContentView;
@property (nonatomic, strong) AbstractActionSheetPicker *actionSheetPicker;
@property (nonatomic, strong) NSDate *selectedDate;
@property (nonatomic, strong) NSDate *selectedTime;
@property (weak, nonatomic) IBOutlet UIButton *birthDateButton;
@property (weak, nonatomic) IBOutlet UIButton *occupationButton;
@property (weak, nonatomic) IBOutlet UILabel *statusBarBackgroundLabel;
@property (weak, nonatomic) IBOutlet UITextField *nameLabel;
@property (weak, nonatomic) IBOutlet UITextField *organizationLabel;
@property (weak, nonatomic) IBOutlet UITextField *emailLabel;
@property (weak, nonatomic) IBOutlet UISegmentedControl *genderSwitch;
@property (weak, nonatomic) IBOutlet UIButton *videoTimeButton;
@property (nonatomic, assign) BOOL isUpdating;
@property (strong, nonatomic) NSArray *videos;

- (IBAction)selectADate:(id)sender;

@end


@implementation SignUpViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    [self configureLayout];
    [self configureNavBar];
    [self fillFieldsWithCustomerInformation];
    
    //Inicialización de datos
    self.moc =  [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    
    if ([[CourseManager sharedInstance] isUserSigned])
    {
        self.title = NSLocalizedString(@"pageUpdateTitleLabelText", nil);
        self.navBar.topItem.title = NSLocalizedString(@"pageUpdateTitleLabelText", nil);
        self.isUpdating = YES;
    }
    else
    {
        self.navBar.topItem.title = NSLocalizedString(@"pageTitleLabelText", nil);
        self.isUpdating = NO;
    }
    
    self.selectedDate = [NSDate date];
    self.selectedTime =[NSDate dateWithTimeIntervalSinceNow:0];
    self.occupations = [NSArray arrayWithObjects:
                        NSLocalizedString(@"job1", nil),
                        NSLocalizedString(@"job2", nil),
                        NSLocalizedString(@"job3", nil),
                        NSLocalizedString(@"job4", nil),
                        NSLocalizedString(@"job5", nil),
                        NSLocalizedString(@"job6", nil),
                        NSLocalizedString(@"job7", nil),
                        NSLocalizedString(@"job8", nil),
                        nil];
    
    self.nameLabel.delegate = self;
    self.organizationLabel.delegate = self;
    self.emailLabel.delegate = self;
}

- (void)viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];
    self.formScrollableView.scrollEnabled = YES;
    [self.formScrollableView setContentSize:CGSizeMake(self.formScrollableView.frame.size.width, self.formContentView.frame.size.height)];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    
    // Dispose of any resources that can be recreated.
    [self setEmailLabel:nil];
    [self setNameLabel:nil];
    [self setOrganizationLabel:nil];
    [self setOccupationButton:nil];
    [self setBirthDateButton:nil];
    [self setVideoTimeButton:nil];
    [self setGenderSwitch:nil];
    [self setView:nil];
}

#pragma mark - IBActions
- (void)signUpButtonTapped {
    if ([self validateForm]){
        NSDateFormatter *dateFormater = [[NSDateFormatter alloc] init];
        [dateFormater setDateStyle:NSDateFormatterMediumStyle];
        NSDate *date = [dateFormater dateFromString:self.birthDateButton.titleLabel.text];
        [CourseManager sharedInstance].currentUser.name = self.nameLabel.text;
        [CourseManager sharedInstance].currentUser.occupation = self.occupationButton.titleLabel.text;
        [CourseManager sharedInstance].currentUser.email = self.emailLabel.text;
        [CourseManager sharedInstance].currentUser.birthDate = date;
        [CourseManager sharedInstance].currentUser.videoAvaliableTime = self.videoTimeButton.titleLabel.text;
        [CourseManager sharedInstance].currentUser.organization = self.organizationLabel.text;
        [CourseManager sharedInstance].currentUser.gender = [self.genderSwitch titleForSegmentAtIndex:self.genderSwitch.selectedSegmentIndex];
        
        [[CourseManager sharedInstance] updateUserInformationWithCompletion:^(NSError *error) {
            if (error){
                [ErrorMessageHelper showErrorMessageWith:NSLocalizedString(@"NetworkError", nil)];
            }
            else if(self.isUpdating) {
                [ErrorMessageHelper showMessageWithTitle:@"" andMessage:NSLocalizedString(@"UpdateSucess", nil)];
            }
            else {
                [self performSegueWithIdentifier:@"signUpVC-tutorialVC" sender:self];
            }
        }];
    }
    else {
        [ErrorMessageHelper showErrorMessageWith:NSLocalizedString(@"InvalidForm", nil)];
    }
}

#pragma mark - UITextField delegates

-(void) touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event{
    [self.emailLabel resignFirstResponder];
    [self.organizationLabel resignFirstResponder];
    [self.nameLabel resignFirstResponder];
    
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return NO;
}

#pragma mark - Private methods

- (void) configureLayout
{
    //estilo para las label de título
    [self.nameTitleLabel setTextColor:backgroudLabel];
    [self.nameTitleLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:18]];
    [self.occupationTitleLabel setTextColor:backgroudLabel];
    [self.occupationTitleLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:18]];
    [self.genderTitleLabel setTextColor:backgroudLabel];
    [self.genderTitleLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:18]];
    [self.emailTitleLabel setTextColor:backgroudLabel];
    [self.emailTitleLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:18]];
    [self.birthTitleLabel setTextColor:backgroudLabel];
    [self.birthTitleLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:18]];
    [self.organizationTitleLabel setTextColor:backgroudLabel];
    [self.organizationTitleLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:18]];
    [self.timeVideoTitleLabel setTextColor:backgroudLabel];
    [self.timeVideoTitleLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:18]];
    
    self.nameTitleLabel.text = NSLocalizedString(@"nameTitleLabelText", nil);
    self.occupationTitleLabel.text = NSLocalizedString(@"occupationTitleLabelText", nil);
    self.genderTitleLabel.text = NSLocalizedString(@"genderTitleLabelText", nil);
    self.emailTitleLabel.text = NSLocalizedString(@"emailTitleLabelText", nil);
    self.birthTitleLabel.text = NSLocalizedString(@"birthTitleLabelText", nil);
    self.organizationTitleLabel.text = NSLocalizedString(@"organizationTitleLabelText", nil);
    self.timeVideoTitleLabel.text = NSLocalizedString(@"timeVideoTitleLabelText", nil);
    
    //Estilos para la navbar
    self.statusBarBackgroundLabel.backgroundColor = navbarBackground;
    
    //bordes de los text field
    [self.nameLabel.layer setBorderColor:[UIColor lightGrayColor].CGColor];
    [self.organizationLabel.layer setBorderColor:[UIColor lightGrayColor].CGColor];
    [self.emailLabel.layer setBorderColor:[UIColor lightGrayColor].CGColor];
    //Fuentes para los text Field
    [self.nameLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:15]];
    [self.organizationLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:15]];
    [self.emailLabel setFont:[UIFont fontWithName:@"Aleo-Regular" size:15]];
    
    //Estilos de los botones que simulan textFields
    [self.birthDateButton.layer setBorderWidth:0.5f];
    self.birthDateButton.layer.cornerRadius = 5.0f;
    [self.birthDateButton.layer setBorderColor:[UIColor lightGrayColor].CGColor];
    [self.videoTimeButton.layer setBorderWidth:0.5f];
    self.videoTimeButton.layer.cornerRadius = 5.0f;
    [self.videoTimeButton.layer setBorderColor:[UIColor lightGrayColor].CGColor];
    [self.occupationButton.layer setBorderWidth:0.5f];
    self.occupationButton.layer.cornerRadius = 5.0f;
    [self.occupationButton.layer setBorderColor:[UIColor lightGrayColor].CGColor];
    
    self.videoTimeButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
    [self.videoTimeButton setTitleEdgeInsets:UIEdgeInsetsMake(0, 8, 0, 0)];
    self.occupationButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
    [self.occupationButton setTitleEdgeInsets:UIEdgeInsetsMake(0, 8, 0, 0)];
    self.birthDateButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
    [self.birthDateButton setTitleEdgeInsets:UIEdgeInsetsMake(0, 8, 0, 0)];
    
    [self.genderSwitch setTitle:NSLocalizedString(@"male", nil) forSegmentAtIndex:0];
    [self.genderSwitch setTitle:NSLocalizedString(@"female", nil) forSegmentAtIndex:1];
}

- (void)configureNavBar
{
    UIBarButtonItem *rightBarButtonItem = [[UIBarButtonItem alloc]initWithTitle:NSLocalizedString(@"doneBarButton", nil) style:UIBarButtonItemStyleDone  target:self action:@selector(signUpButtonTapped)];
    self.navBar.topItem.rightBarButtonItem = rightBarButtonItem;
    self.navBar.backgroundColor = navbarBackground;
    self.navBar.tintColor = [UIColor whiteColor];
}

- (void) fillFieldsWithCustomerInformation{
    NSString *date = [NSDateFormatter localizedStringFromDate:[CourseManager sharedInstance].currentUser.birthDate
                                                    dateStyle:NSDateFormatterMediumStyle
                      
                                                    timeStyle:NSDateFormatterNoStyle];
    NSInteger field = ([[CourseManager sharedInstance].currentUser.gender isEqualToString:@"Male"] || [[CourseManager sharedInstance].currentUser.gender isEqualToString:@"Hombre"]) ? 1 : 0;
    self.genderSwitch.selectedSegmentIndex = field;
    self.genderSwitch.enabled = YES;
    self.nameLabel.text = [CourseManager sharedInstance].currentUser.name;
    [self.occupationButton setTitle:[CourseManager sharedInstance].currentUser.occupation forState:UIControlStateNormal];
    [self.birthDateButton   setTitle:date forState:UIControlStateNormal];
    [self.videoTimeButton setTitle:[CourseManager sharedInstance].currentUser.videoAvaliableTime forState:UIControlStateNormal];
    self.emailLabel.text = [CourseManager sharedInstance].currentUser.email;
    self.organizationLabel.text = [CourseManager sharedInstance].currentUser.organization;
}

- (IBAction)selectADate:(UIControl *)sender{
    _actionSheetPicker = [[ActionSheetDatePicker alloc] initWithTitle:@"" datePickerMode:UIDatePickerModeDate selectedDate:self.selectedDate target:self action:@selector(dateWasSelected:element:) origin:sender];
    self.actionSheetPicker.hideCancel = YES;
    [self.actionSheetPicker showActionSheetPicker];
}

- (void)dateWasSelected:(NSDate *)selectedDate element:(id)element {
    
   NSString *date = [NSDateFormatter localizedStringFromDate:selectedDate
                                                            dateStyle:NSDateFormatterMediumStyle
                                                            timeStyle:NSDateFormatterNoStyle];
    [self.birthDateButton setTitle:date forState:UIControlStateNormal];

}

- (IBAction)selectAnOccupation:(UIControl *)sender{
    _actionSheetPicker =[[ActionSheetStringPicker alloc] initWithTitle:@"" rows:self.occupations initialSelection:self.selectedIndex target:self successAction:@selector(OccupationWasSelected:element:) cancelAction:nil origin:sender];
    self.actionSheetPicker.hideCancel = YES;
    [self.actionSheetPicker showActionSheetPicker];
}

- (void)OccupationWasSelected:(NSNumber *)selectedIndex element:(id)element {
    self.selectedIndex = [selectedIndex intValue];
    [self.occupationButton setTitle:[self.occupations objectAtIndex:self.selectedIndex] forState:UIControlStateNormal];
}

- (void)actionPickerCancelled:(id)sender {
    NSLog(@"Delegate has been informed that ActionSheetPicker was cancelled");
}

- (IBAction)selectATime:(UIControl *)sender{
    _actionSheetPicker = [[ActionSheetDatePicker alloc] initWithTitle:@"" datePickerMode:UIDatePickerModeTime selectedDate:self.selectedTime target:self action:@selector(timeWasSelected:element:) origin:sender];
    self.actionSheetPicker.hideCancel = YES;
    [self.actionSheetPicker showActionSheetPicker];
}

- (void)timeWasSelected:(NSDate *)selectedTime element:(id)element {
    
    self.selectedTime = selectedTime;
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init] ;
    [formatter setDateFormat:@"HH:mm"];
    NSString *timeString = [formatter stringFromDate:selectedTime];
    [self.videoTimeButton setTitle:timeString forState:UIControlStateNormal];
  }


- (BOOL) validateForm{
    BOOL result = true;
    
    if(!self.nameLabel.text || self.nameLabel.text.length <= 0){
        [self.nameLabel.layer setBorderColor:navbarBackground.CGColor];
        self.nameLabel.text = nil;
        self.nameLabel.placeholder  = NSLocalizedString(@"nameError", nil);
        [self.nameLabel.layer setBorderWidth:1.5f];
        self.nameLabel.layer.cornerRadius = 5.0f;
        [self.nameLabel setValue:[UIColor blackColor] forKeyPath:@"_placeholderLabel.textColor"];

        result = false;
    }
    
    if(!self.occupationButton.titleLabel.text || self.occupationButton.titleLabel.text.length <= 0){
        [self.occupationButton.layer setBorderColor:navbarBackground.CGColor];
        [self.occupationButton setTitle:NSLocalizedString(@"occupationError", nil) forState:UIControlStateNormal];
        [self.occupationButton.layer setBorderWidth:1.5f];
        result = false;
    }

    if(!self.birthDateButton.titleLabel.text || self.birthDateButton.titleLabel.text.length == 0){
        [self.birthDateButton.layer setBorderColor:navbarBackground.CGColor];
        [self.birthDateButton setTitle:NSLocalizedString(@"birthDateError", nil) forState:UIControlStateNormal];
        [self.birthDateButton.layer setBorderWidth:1.5f];
        result = false;
    }
    
    if(!self.videoTimeButton.titleLabel.text || self.birthDateButton.titleLabel.text.length == 0){
        [self.videoTimeButton.layer setBorderColor:navbarBackground.CGColor];
        [self.videoTimeButton setTitle:NSLocalizedString(@"videoTimeError", nil) forState:UIControlStateNormal];
        [self.videoTimeButton.layer setBorderWidth:1.5];
        result = false;
    }
   
    if(!self.emailLabel.text || self.emailLabel.text.length == 0 || ![self NSStringIsValidEmail:self.emailLabel.text]){
        [self.emailLabel.layer setBorderColor:navbarBackground.CGColor];
        self.emailLabel.text = nil;
        self.emailLabel.placeholder = NSLocalizedString(@"emailError", nil);
        [self.emailLabel.layer setBorderWidth:1.5f];
        self.emailLabel.layer.cornerRadius = 5.0f;
        [self.emailLabel setValue:[UIColor blackColor] forKeyPath:@"_placeholderLabel.textColor"];

        result = false;
    }
    
    if(!self.organizationLabel.text || self.organizationLabel.text.length == 0){
        [self.organizationLabel.layer setBorderColor:navbarBackground.CGColor];
        self.organizationLabel.text = nil;
        self.organizationLabel.placeholder = NSLocalizedString(@"organizationError", nil);
        [self.organizationLabel setValue:[UIColor blackColor] forKeyPath:@"_placeholderLabel.textColor"];

        [self.organizationLabel.layer setBorderWidth:1.5f];
        self.organizationLabel.layer.cornerRadius = 5.0f;

        result = false;
    }

    return result;
}

-(BOOL) NSStringIsValidEmail:(NSString *)checkString
{
    BOOL stricterFilter = YES;
    NSString *stricterFilterString = @"[A-Z0-9a-z\\._%+-]+@([A-Za-z0-9-]+\\.)+[A-Za-z]{2,4}";
    NSString *laxString = @".+@([A-Za-z0-9]+\\.)+[A-Za-z]{2}[A-Za-z]*";
    NSString *emailRegex = stricterFilter ? stricterFilterString : laxString;
    NSPredicate *emailTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", emailRegex];
    return [emailTest evaluateWithObject:checkString];
}

@end

