//
//  TestQuestion.m
//  IOS Learning Plaform
//
//  Created by Ruben Domínguez on 11/9/16.
//  Copyright © 2016 TheAgileMonkeys. All rights reserved.
//

#import "TestQuestion.h"
#import <Parse/Parse.h>

@interface TestQuestion ()
@property (nonatomic, copy) NSString *spanish_explanation;
@property (nonatomic, copy) NSString *spanish_statement;
@property (nonatomic, copy) NSString *english_explanation;
@property (nonatomic, copy) NSString *english_statement;
@end

@implementation TestQuestion

- (instancetype)initWithDictionary:(NSDictionary *)dictionary
{
    self = [super init];
    
    if(self) {
        _modelId = dictionary[@"objectId"];
        _answer  = dictionary[@"answer"];
        _spanish_statement = dictionary[@"spanish_statement"];
        _spanish_explanation = dictionary[@"spanish_explanation"];
        _english_statement = dictionary[@"english_statement"];
        _english_explanation  = dictionary[@"english_explanation"];        
    }
    
    return self;
}

- (void)markAsWrongForUser:(User *)user withCompletion:(void (^)(NSError *))completion
{
    PFObject *answer = [PFObject objectWithClassName:@"Test_Answer"];
    answer[@"user"] = user.modelId ;
    answer[@"question"]= self.modelId;
    answer[@"date"] = [NSDate date];
    answer[@"isCorrect"] = @NO;
    
    [answer saveInBackgroundWithBlock:^(BOOL succeeded, NSError *error) {
        if (completion) {
            completion(error);
        }
    }];
}

- (void)markAsCorrectForUser:(User *)user withCompletion:(void (^)(NSError *))completion
{
    PFObject *answer = [PFObject objectWithClassName:@"Test_Answer"];
    answer[@"user"] = user.pfObject ;
    answer[@"question"]= self.modelId;
    answer[@"date"] = [NSDate date];
    answer[@"isCorrect"] = @YES;

    [answer saveInBackgroundWithBlock:^(BOOL succeeded, NSError *error) {
        if (completion) {
            completion(error);
        }
    }];
    
}

- (NSString *)explanation
{
    return ([[[NSLocale preferredLanguages] objectAtIndex:0] isEqualToString:@"es-ES"]) ? self.spanish_explanation : self.english_explanation;
}

- (NSString *)statement
{
        return ([[[NSLocale preferredLanguages] objectAtIndex:0] isEqualToString:@"es-ES"]) ? self.spanish_statement : self.english_statement;
}
@end
