//
//  TestViewController.m
//  IOS Learning Plaform
//
//  Created by Rubén on 18/11/13.
//  Copyright (c) 2013 TheAgileMonkeys. All rights reserved.
//

#import "TestViewController.h"
#import "TestProgresBar.h"
#import "QuestionConfirmViewController.h"
#import "WrongAnswerViewController.h"
#import "FinishTestViewController.h"
#import "test.h"
#import "TestQuestion.h"

@interface TestViewController () <UITableViewDelegate, UITableViewDataSource>
@property (weak, nonatomic) IBOutlet UITableView *questionsTableVIew;
@property (strong, nonatomic) NSArray *questions;
@property (nonatomic, assign) BOOL responseWasCorrect;
@property (nonatomic) NSUInteger index;
@property (nonatomic) NSUInteger n_questions;
@end

@implementation TestViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
    self.navigationController.navigationBar.backgroundColor = navbarBackground;
    self.title = NSLocalizedString(@"testViewPageTitle", nil);
    self.questionsTableVIew.delegate = self;
    self.questionsTableVIew.dataSource = self;
    self.questionsTableVIew.scrollEnabled = NO;
    self.index = 0;
    self.n_questions = 10;
   [self prepareInfo];
}

#pragma mark - Table view data source
- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
   return (self.questions && ([self.questions count] > 0)) ? 128.0f : 0.0f;}

- (UIView *) tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    if (self.questions && ([self.questions count] > 0)){
        
        UIView *view = [[UIView alloc] initWithFrame:CGRectMake(0, 0, tableView.frame.size.width, 128)];
        
        UILabel *statement = [[UILabel alloc] initWithFrame:CGRectMake(2.5, 0, tableView.frame.size.width-5, 128)];
        statement.numberOfLines = 0;
        statement.font = [UIFont fontWithName:@"Aleo-Bold" size:18.0];
        statement.textColor = navbarBackground;
        statement.textAlignment = NSTextAlignmentCenter;
                
        TestQuestion *currentQuestion = self.questions[self.index];
        statement.text = currentQuestion.statement;
        
        [view addSubview:statement];
        
        
        return view;
    }

    return nil;
}

- (CGFloat) tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section{
    
   return (self.questions && ([self.questions count] > 0)) ? 200.0f : 0.01f;
}

- (UIView *) tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section{
    if (self.questions && ([self.questions count] > 0)){
        NSInteger solved = self.n_questions - [self.questions count] + self.index;
        UIView *view = [[UIView alloc] initWithFrame:CGRectMake(0, 0, tableView.frame.size.width, 128)];
        TestProgresBar *footerView = [[TestProgresBar alloc] initWithFrame:CGRectMake(10, 87, tableView.frame.size.width -20, 20)];
        footerView.backgroundColor = [UIColor whiteColor];
        footerView.index =  self.n_questions - [self.questions count] + self.index;
        footerView.n_questions = self.n_questions;
        footerView.width = tableView.frame.size.width - 20;
        
        UILabel * count =[[UILabel alloc]initWithFrame:CGRectMake(2.5, 107, tableView.frame.size.width - 5, 20)];
        count.text = [NSString stringWithFormat:@"[%ld of %lu]" , solved + 1, (unsigned long)self.n_questions];
        count.numberOfLines = 1;
        count.font = [UIFont fontWithName:@"Aleo-Regular" size:15.0];
        count.textColor = backgroudLabel;
        count.textAlignment = NSTextAlignmentCenter;
        [view addSubview:count];
        [view addSubview:footerView];

        return view;
    }
    else return nil;
    
}
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return (self.questions && ([self.questions count] > 0)) ? 2 : 0;
}



- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"testCell" forIndexPath:indexPath];
    
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}


// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{

    TestQuestion *currentQuestion = self.questions[self.index];
    BOOL selected_answer = (indexPath.row == 0) ? YES : NO;

    if (currentQuestion.answer == selected_answer) {
        
        [currentQuestion markAsCorrectForUser:[CourseManager sharedInstance].currentUser withCompletion:^(NSError *error) {
            self.responseWasCorrect = YES;
            [self nextQuestion];
            if(error) {
                [ErrorMessageHelper showErrorMessageWith:NSLocalizedString(@"NetworkError", nil)];
            }
            else {
                self.responseWasCorrect = YES;
                [self nextQuestion];
            }
        }];
    }
    
    else{
        [currentQuestion markAsWrongForUser:[CourseManager sharedInstance].currentUser withCompletion:^(NSError *error) {
            self.responseWasCorrect = NO;
            WrongAnswerViewController *confirmationController = [self.storyboard instantiateViewControllerWithIdentifier:@"wrongAnswerVC"];
            confirmationController.message = currentQuestion.explanation;
            confirmationController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
            [self presentViewController:confirmationController animated:YES completion:nil];
            
            if (!error) {
                self.responseWasCorrect = NO;
                WrongAnswerViewController *confirmationController = [self.storyboard instantiateViewControllerWithIdentifier:@"wrongAnswerVC"];
                confirmationController.message = currentQuestion.explanation;
                confirmationController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
                [self presentViewController:confirmationController animated:YES completion:nil];
            }
            else{
                [ErrorMessageHelper showErrorMessageWith:NSLocalizedString(@"NetworkError", nil)];
            }
        }];
    }
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    cell.textLabel.textAlignment = NSTextAlignmentCenter;
    if (indexPath.row == 0) {
        cell.textLabel.text = NSLocalizedString(@"trueTestAnswer", nil);
    }
    else{
        cell.textLabel.text = NSLocalizedString(@"falseTestAnswer", nil);

    }
}

#pragma mark private
- (void) nextQuestion{

    if (self.responseWasCorrect){
        if (self.index < ([self.questions count]-1)) {
            self.index ++;
            QuestionConfirmViewController *confirmationController = [self.storyboard instantiateViewControllerWithIdentifier:@"confirmationQuestionVC"];
            confirmationController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
            [self presentViewController:confirmationController animated:YES completion:nil];
            [self.questionsTableVIew reloadData];
        }
        else{
            [[NSUserDefaults standardUserDefaults]setBool:YES forKey:@"isTestFinished"];
            [[NSUserDefaults standardUserDefaults] synchronize];
            FinishTestViewController *finishTestnotice = [self.storyboard instantiateViewControllerWithIdentifier:@"finishTestVC"];
            finishTestnotice.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
            [self presentViewController:finishTestnotice animated:YES completion:nil];
        }
        
    }
}

- (void) prepareInfo{
    
    [Test getTestQuestionsForUser:[CourseManager sharedInstance].currentUser withCompletion:^(NSArray *questions, NSError *error) {
        if (error) {
            [ErrorMessageHelper showErrorMessageWith:NSLocalizedString(@"NetworkError", nil)];
        }
        else {
            self.questions = questions;
            [self.questionsTableVIew reloadData];
        }
    }];
}

@end
