//
//  ToDoViewController.m
//  IOS Learning Plaform
//
//  Created by Rubén on 08/10/13.
//  Copyright (c) 2013 TheAgileMonkeys. All rights reserved.
//

#import "ToDoViewController.h"
#import "EditTodoViewController.h"
#import "ToDo.h"
#import <Parse/Parse.h>

@interface ToDoViewController () <NSFetchedResultsControllerDelegate>
@property (nonatomic, weak) NSManagedObjectContext *moc;
@property (strong, nonatomic) NSFetchedResultsController *fetchedResultsController;
@property (weak, nonatomic)  ToDo *actual_todo;
@end


@implementation ToDoViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view
    
    self.moc =  [(AppDelegate *)[[UIApplication sharedApplication] delegate] managedObjectContext];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    NSError *error = nil;
    if (![self.fetchedResultsController performFetch:&error])
    {
        NSLog(@"ERROR = %@", [error localizedDescription]);
    }
}


- (void) viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    
    self.navigationController.navigationBar.topItem.title = NSLocalizedString(@"toDoListPageTitle", nil);
    UIBarButtonItem *rightBarButtonItem = [[UIBarButtonItem alloc]initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addToDo)];
    self.navigationController.navigationBar.topItem.rightBarButtonItem = rightBarButtonItem;
}

- (void) viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    self.navigationController.navigationBar.topItem.rightBarButtonItem = nil;
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


#pragma mark - Table View

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return 0.0f;
}


- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    ToDo *aTodo = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    NSAttributedString *attributedText = [[NSAttributedString alloc] initWithString:aTodo.body attributes:@{NSFontAttributeName:[UIFont systemFontOfSize:17.0]}];
    CGRect rect = [attributedText boundingRectWithSize:(CGSize){280.0, CGFLOAT_MAX}
                                               options:NSStringDrawingUsesLineFragmentOrigin
                                               context:nil];
    return rect.size.height + 40.0f;
}


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [[self.fetchedResultsController sections] count];
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    id <NSFetchedResultsSectionInfo> sectionInfo = [self.fetchedResultsController sections][section];
    return [sectionInfo numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"toDoCell" forIndexPath:indexPath];
    
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}


- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return YES;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        [self.moc deleteObject:[self.fetchedResultsController objectAtIndexPath:indexPath]];
        
        NSError *error = nil;
        if (![self.moc save:&error]) {
            // Replace this implementation with code to handle the error appropriately.
            // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        }
    }
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //    EditTodoViewController *createToDoViewController = [self.storyboard instantiateViewControllerWithIdentifier:(@"EditTodoViewController")];
    //
    //    createToDoViewController.moc = self.moc;
    //    createToDoViewController.object = [self.fetchedResultsController objectAtIndexPath:indexPath];
    //    [self.navigationController pushViewController:createToDoViewController animated:YES];
    
    ToDo *object = [self.fetchedResultsController objectAtIndexPath:indexPath];
    PFObject *todo = [PFObject objectWithClassName:@"Todo_Analytics"];
    todo.objectId = self.actual_todo.parseId;
    if ([object.done boolValue]){
        todo[@"done"] = @NO;
        object.done = @NO;
    }
    else{
        todo[@"done"] = @YES;
        object.done = @YES;
        
    }
    [todo saveInBackgroundWithBlock:^(BOOL succeeded, NSError *error) {
        if (!error) {
            NSError *error;
            [self.moc save:&error];
        }
        else{
            UIAlertView *errorAlert = [[UIAlertView alloc] initWithTitle:@"Error"
                                                                 message:NSLocalizedString(@"NetworkError", nil)
                                                                delegate:nil
                                                       cancelButtonTitle:@"OK"
                                                       otherButtonTitles:nil];
            [errorAlert show];
            
        }
    }];
    
    
}


#pragma mark - Fetched results controller

- (NSFetchedResultsController *)fetchedResultsController
{
    if (_fetchedResultsController != nil) {
        return _fetchedResultsController;
    }
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"ToDo" inManagedObjectContext:self.moc];
    [fetchRequest setEntity:entity];
    
    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"done" ascending:YES];
    NSSortDescriptor *sortDescriptor2 = [[NSSortDescriptor alloc] initWithKey:@"date" ascending:YES];
    
    NSArray *sortDescriptors = @[sortDescriptor, sortDescriptor2];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:self.moc sectionNameKeyPath:@"done" cacheName:nil];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
    NSError *error = nil;
    if (![self.fetchedResultsController performFetch:&error]) {
        // Replace this implementation with code to handle the error appropriately.
        // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }
    
    return _fetchedResultsController;
}

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
           atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type
{
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    UITableView *tableView = self.tableView;
    
    switch(type) {
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:@[newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView endUpdates];
}


- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    ToDo *object = [self.fetchedResultsController objectAtIndexPath:indexPath];
    UITapGestureRecognizer *tap;
    //format Date
    NSString *strDate = [NSDateFormatter localizedStringFromDate:object.date
                                                       dateStyle:NSDateFormatterMediumStyle
                                                       timeStyle:NSDateFormatterNoStyle];
    NSMutableAttributedString *strDateFormat;
    strDateFormat = [[NSMutableAttributedString alloc] initWithString:[strDate stringByAppendingString:[NSString stringWithFormat:@" %@",object.body]]];
    
    if ([object.done boolValue]) {
        [strDateFormat addAttribute: NSForegroundColorAttributeName value: backgroudLabel range: NSMakeRange(0, strDateFormat.length)];
        cell.imageView.image = [UIImage imageNamed:@"icon_todo2.png"];
        tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(markLikeNotDone:)];
    }
    else {
        [strDateFormat addAttribute: NSForegroundColorAttributeName value: navbarBackground range: NSMakeRange(0, strDate.length)];
        cell.imageView.image = [UIImage imageNamed:@"icon_todo1.png"];
        tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(markLikeDone:)];
    }
    
    [cell.textLabel setAttributedText:strDateFormat];
    cell.textLabel.numberOfLines = 0;
    [cell.imageView setUserInteractionEnabled:YES];
    [tap setNumberOfTouchesRequired:1];
    [tap setNumberOfTapsRequired:1];
    self.actual_todo = object;
    [cell.imageView addGestureRecognizer:tap];
    
}


- (void)markLikeDone:(UITapGestureRecognizer *)gestureRecognizer
{
    PFObject *todo = [PFObject objectWithClassName:@"Todo_Analytics"];
    todo.objectId = self.actual_todo.parseId;
    todo[@"done"] = @YES;
    self.actual_todo.done = @YES;
    [todo saveInBackgroundWithBlock:^(BOOL succeeded, NSError *error) {
        if (!error) {
            NSError *error;
            [self.moc save:&error];
        }
        else{
            UIAlertView *errorAlert = [[UIAlertView alloc] initWithTitle:@"Error"
                                                                 message:NSLocalizedString(@"NetworkError", nil)
                                                                delegate:nil
                                                       cancelButtonTitle:@"OK"
                                                       otherButtonTitles:nil];
            [errorAlert show];
            
        }
    }];
}

- (void)markLikeNotDone:(UITapGestureRecognizer *)gestureRecognizer
{
    PFObject *todo = [PFObject objectWithClassName:@"Todo_Analytics"];
    todo.objectId = self.actual_todo.parseId;
    todo[@"done"] = @NO;
    self.actual_todo.done = @NO;
    [todo saveInBackgroundWithBlock:^(BOOL succeeded, NSError *error) {
        if (!error) {
            NSError *error;
            [self.moc save:&error];
        }
        else{
            UIAlertView *errorAlert = [[UIAlertView alloc] initWithTitle:@"Error"
                                                                 message:NSLocalizedString(@"NetworkError", nil)
                                                                delegate:nil
                                                       cancelButtonTitle:@"OK"
                                                       otherButtonTitles:nil];
            [errorAlert show];
            
        }
    }];
}


#pragma mark actions
- (void)addToDo {
    EditTodoViewController *createToDoViewController = [self.storyboard instantiateViewControllerWithIdentifier:(@"EditTodoViewController")];
    createToDoViewController.object = nil;
    createToDoViewController.moc = self.moc;
    createToDoViewController.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
    [self.navigationController pushViewController:createToDoViewController animated:YES];
}

@end
